package com.bweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.BsDiscount;
import com.hfkj.entity.BsDiscountStockCode;
import com.hfkj.entity.BsMerchant;
import com.hfkj.model.ResponseData;
import com.hfkj.service.BsDiscountService;
import com.hfkj.service.BsDiscountStockBatchService;
import com.hfkj.service.BsDiscountStockCodeService;
import com.hfkj.service.BsMerchantService;
import com.hfkj.sysenum.DiscountStatusEnum;
import com.hfkj.sysenum.DiscountTypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value = "/discountStock")
@Api(value = "优惠券库存管理")
public class BsDiscountStockController {

    private static Logger log = LoggerFactory.getLogger(BsDiscountStockController.class);
    @Resource
    private BsDiscountStockBatchService discountStockBatchService;

    @Resource
    private BsDiscountStockCodeService discountStockCodeService;

    @RequestMapping(value="/addStock",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加优惠券库存")
    public ResponseData addStock(@RequestBody JSONObject body) {
        try {

            if (body == null
                || StringUtils.isBlank(body.getString("discountNo"))
                || body.getInteger("stockCount") == null) {
                log.error("BsDiscountController -> editDiscount() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            discountStockBatchService.addStock(body.getString("discountNo"), body.getInteger("stockCount"));

            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("BsDiscountStockController -> addStock() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryStockBatchList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询库存批次")
    public ResponseData queryStockBatchList(@RequestParam(name = "discountNo", required = false) String discountNo,
                                            @RequestParam(name = "discountName", required = false) String discountName,
                                            @RequestParam(name = "batchNo", required = false) String batchNo,
                                            @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                            @RequestParam(name = "pageSize", required = true) Integer pageSize) {
        try {
            Map<String,Object> param = new HashMap<>();
            param.put("discountNo", discountNo);
            param.put("discountName", discountName);
            param.put("batchNo", batchNo);

            PageHelper.startPage(pageNum,pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(discountStockBatchService.getStockBatchList(param)));

        } catch (Exception e) {
            log.error("BsDiscountStockController -> queryStockBatchList() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryStockBatchDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询库存批次详情")
    public ResponseData queryStockBatchDetail(@RequestParam(name = "batchNo", required = true) String batchNo) {
        try {

            return ResponseMsgUtil.success(discountStockBatchService.getStockBatchDetail(batchNo));

        } catch (Exception e) {
            log.error("BsDiscountStockController -> queryStockBatchDetail() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }



    @RequestMapping(value="/queryStockBatchCodeList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询库存批次优惠券code")
    public ResponseData queryStockBatchCodeList(@RequestParam(name = "codeId", required = false) String codeId,
                                                @RequestParam(name = "discountNo", required = false) String discountNo,
                                                @RequestParam(name = "batchNo", required = false) String batchNo,
                                                @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                                @RequestParam(name = "pageSize", required = true) Integer pageSize) {
            try {
            Map<String,Object> param = new HashMap<>();
            param.put("codeId", codeId);
            param.put("discountNo", discountNo);
            param.put("batchNo", batchNo);

            PageHelper.startPage(pageNum,pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(discountStockCodeService.getCodeList(param)));

        } catch (Exception e) {
            log.error("BsDiscountStockController -> queryStockBatchCodeList() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
