package com.cweb.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.BsAgentPrice;
import com.hfkj.entity.BsGasOilPrice;
import com.hfkj.model.GasListModel;
import com.hfkj.model.GasModel;
import com.hfkj.model.ResponseData;
import com.hfkj.service.agent.BsAgentPriceService;
import com.hfkj.service.gas.BsGasService;
import com.hfkj.sysenum.agent.AgentPriceTypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;

@Controller
@RequestMapping(value = "/gas")
@Api(value = "油站管理")
public class BsGasController {
    private static Logger log = LoggerFactory.getLogger(BsGasController.class);

    @Resource
    private BsGasService gasService;
    @Resource
    private BsAgentPriceService agentPriceService;

    @RequestMapping(value="/oilPriceCompute",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "油价计算")
    public ResponseData oilPriceCompute(@RequestParam(name = "agentMerId", required = false) Long agentMerId,
                                        @RequestParam(name = "refuelPrice", required = true) BigDecimal refuelPrice,
                                        @RequestParam(name = "merNo", required = true) String merNo,
                                        @RequestParam(name = "oilNo", required = true) String oilNo) {
        try {

            return ResponseMsgUtil.success(gasService.refuelPriceCompute(refuelPrice,merNo,oilNo,agentMerId));

        } catch (Exception e) {
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryGasList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询加油站列表")
    public ResponseData queryGasList(@RequestParam(name = "agentMerId", required = false) Long agentMerId,
                                     @RequestParam(name = "gasName", required = false) String gasName,
                                     @RequestParam(name = "distanceLimit", required = false) Integer distanceLimit,
                                     @RequestParam(name = "oilNo", required = true) String oilNo,
                                     @RequestParam(name = "longitude", required = true) String longitude,
                                     @RequestParam(name = "latitude", required = true) String latitude,
                                     @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                     @RequestParam(name = "pageSize", required = true) Integer pageSize
    ) {
        try {

            PageHelper.startPage(pageNum,pageSize);
            PageInfo<GasListModel> pageInfo = new PageInfo<>(gasService.getGasList(longitude, latitude, oilNo, gasName, distanceLimit));

            for (GasListModel gas : pageInfo.getList()) {
                BigDecimal discount = new BigDecimal("100");
                if (agentMerId != null) {
                    // 查询代理价格
                    BsAgentPrice agentPrice = agentPriceService.getDetail(agentMerId, gas.getOilNo());
                    if (agentPrice != null) {
                        discount = agentPrice.getPriceRate();
                    }
                } else {
                    // 查询平台价格
                    BsAgentPrice agentPrice = agentPriceService.getDetail(AgentPriceTypeEnum.type1, gas.getMerNo(), gas.getOilNo());
                    if (agentPrice != null) {
                        discount = agentPrice.getPriceRate();
                    }
                }
                gas.setPriceVip(gas.getPriceGun().multiply(discount.divide(new BigDecimal("100"))).setScale(2, BigDecimal.ROUND_HALF_UP));
            }
            return ResponseMsgUtil.success(pageInfo);

        } catch (Exception e) {
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryGasDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询加油站详情")
    public ResponseData queryGasDetail(@RequestParam(name = "merNo", required = true) String merNo,
                                       @RequestParam(name = "agentMerId", required = false) Long agentMerId) {
        try {
            // 查询油站详情
            GasModel gasDetail = gasService.getGasDetail(merNo);
            if (gasDetail == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的油站");
            }

            for (BsGasOilPrice oilPrice : gasDetail.getGasOilPrice()) {
                BigDecimal discount = new BigDecimal("100");
                if (agentMerId != null) {
                    // 查询代理价格
                    BsAgentPrice agentPrice = agentPriceService.getDetail(agentMerId, oilPrice.getOilNo());
                    if (agentPrice != null) {
                        discount = agentPrice.getPriceRate();
                    }
                } else {
                    // 查询平台价格
                    BsAgentPrice agentPrice = agentPriceService.getDetail(AgentPriceTypeEnum.type1, merNo, oilPrice.getOilNo());
                    if (agentPrice != null) {
                        discount = agentPrice.getPriceRate();
                    }
                }
                oilPrice.setPriceVip(oilPrice.getPriceGun().multiply(discount.divide(new BigDecimal("100"))).setScale(2, BigDecimal.ROUND_HALF_UP));
            }

            return ResponseMsgUtil.success(gasDetail);

        } catch (Exception e) {
            return ResponseMsgUtil.exception(e);
        }
    }

}
