package com.cweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.hfkj.common.alipay.AlipayUtils;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.HttpsUtils;
import com.hfkj.common.utils.MemberValidateUtil;
import com.hfkj.common.utils.RedisUtil;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.model.ResponseData;
import com.hfkj.model.SecUserSessionObject;
import com.hfkj.model.UserSessionObject;
import com.hfkj.service.user.BsUserService;
import com.hfkj.sysenum.user.UserLoginPlatform;
import com.hfkj.sysenum.user.UserLoginType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @className: BsUserController
 * @author: HuRui
 * @date: 2024/6/11
 **/
@Controller
@RequestMapping(value = "/user")
@Api(value = "登录业务")
public class BsUserController {
    private static Logger log = LoggerFactory.getLogger(BsUserController.class);
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private UserCenter userCenter;
    @Resource
    private BsUserService userService;

    @RequestMapping(value = "/login", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "登录并注册")
    public ResponseData login(@RequestBody JSONObject body) {
        try {
            if (body == null
                    || StringUtils.isBlank(body.getString("platform"))
                    || StringUtils.isBlank(body.getString("type"))
                    || StringUtils.isBlank(body.getString("phone"))
            ) {
                log.error("LoginController --> phone() error!", "请求参数校验失败");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }
            String phone = body.getString("phone");

            // 客户端
            UserLoginPlatform platform = UserLoginPlatform.getDataByType(body.getString("platform"));
            if (platform == null) {
                log.error("LoginController --> phone() error!", "未知客户端");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知客户端");
            }
            // 校验手机号格式
            if (!MemberValidateUtil.validatePhone(phone)) {
                log.error("LoginController --> phone() error!", "请输入正确的手机号");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "请输入正确的手机号");
            }
            // 登录类型
            UserLoginType loginType = UserLoginType.getDataByType(body.getString("type"));
            if (loginType == null) {
                log.error("LoginController --> phone() error!", "未知登录类型；" + body.getString("type"));
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知登录类型");
            }

            // 账户密码登录
            if (body.getString("type").equals(UserLoginType.SMS.getCode())) {
                if (StringUtils.isBlank(body.getString("smsCode"))) {
                    log.error("LoginController --> phone() error!", "请输入短信验证码");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "请输入短信验证码");
                }
                // 手机号的验证码
                Object phoneCodeObject = redisUtil.get("SMS_CODE:" + phone);
                if (phoneCodeObject == null) {
                    log.error("LoginController --> phone() error!", "短信验证码错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "短信验证码错误");
                }
                if (!body.getString("smsCode").equals(phoneCodeObject.toString())) {
                    log.error("LoginController --> phone() error!", "短信验证码错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "短信验证码错误");
                }
                redisUtil.del("SMS_CODE:" + phone);

                Map<String, Object> other = new HashMap<>();
                other.put("wxMpOpenId", body.getString("wxMpOpenId"));
                other.put("wxMaOpenId", body.getString("wxMaOpenId"));
                return ResponseMsgUtil.success(userService.login(platform, loginType, phone, other));
            }

            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "登录失败");

        } catch (Exception e) {
            log.error("LoginController --> phone() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/queryUser", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询用户信息")
    public ResponseData queryUser() {
        try {

            return ResponseMsgUtil.success(userCenter.getSessionModel(UserSessionObject.class));

        } catch (Exception e) {
            log.error("LoginController --> wechatMaPhone() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/loginOut",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "退出登录")
    public ResponseData loginOut(HttpServletRequest request) {
        try {
            SecUserSessionObject session = userCenter.getSessionModel(SecUserSessionObject.class);
            if (session != null) {
                userCenter.remove(request);
            }
            return ResponseMsgUtil.success("退出成功");

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getH5AccessToken", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "【H5】获取AccessToken")
    public ResponseData getH5AccessToken(@RequestParam(value = "code", required = true) String code) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("appid", "wxa075e8509802f826");
            params.put("secret", "0e606fc1378d35e359fcf3f15570b2c5");
            params.put("code", code);
            params.put("grant_type", "authorization_code");
            return ResponseMsgUtil.success(HttpsUtils.doGet("https://api.weixin.qq.com/sns/oauth2/access_token", params));

        } catch (Exception e) {
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getAlipayUserId",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取支付宝用户id")
    public ResponseData getAlipayUserId(@RequestParam(value = "code", required = true) String code) {
        try {
            AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
            request.setGrantType("authorization_code");
            request.setCode(code);
            AlipaySystemOauthTokenResponse response = AlipayUtils.initClient().certificateExecute(request);
            if(response.isSuccess()){
                return ResponseMsgUtil.success(response);
            }
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "获取失败");

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
