package com.hfkj.service.agent.impl;

import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.dao.BsAgentMerMapper;
import com.hfkj.entity.*;
import com.hfkj.model.AgentMerAllot;
import com.hfkj.service.BsMerchantService;
import com.hfkj.service.agent.BsAgentMerService;
import com.hfkj.service.agent.BsAgentService;
import com.hfkj.service.agent.BsAgentStaffService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.LongStream;

/**
 * @className: BsAgentMerServiceImpl
 * @author: HuRui
 * @date: 2024/6/15
 **/
@Service("agentMerService")
public class BsAgentMerServiceImpl implements BsAgentMerService {
    @Resource
    private BsAgentMerMapper agentMerMapper;
    @Resource
    private BsAgentService agentService;
    @Resource
    private BsAgentMerService agentMerService;
    @Resource
    private BsAgentStaffService agentStaffService;
    @Resource
    private BsMerchantService merchantService;
    @Override
    public void editData(BsAgentMer data) {
        data.setUpdateTime(new Date());
        if (data.getId() == null) {
            data.setCreateTime(new Date());
            agentMerMapper.insert(data);
        } else {
            agentMerMapper.updateByPrimaryKey(data);
        }
    }

    @Override
    public void addData(List<BsAgentMer> dataList) {
        if (!dataList.isEmpty()) {
            agentMerMapper.addPatch(dataList);
        }
    }

    @Override
    public void assignAgent(Long agentId, List<String> merNoArray) {
        // 代理商
        BsAgent agent = agentService.getAgentById(agentId);
        if (agent == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的代理商");
        }

        // 查询代理商的油站
        Map<String,Object> param = new HashMap<>();
        param.put("agentId", agent.getId());
        List<BsAgentMer> agentMerList = getList(param);

        List<BsAgentMer> dataList = new ArrayList<>();
        BsAgentMer agentMer;
        for (String merNo : merNoArray) {
            // 查询门店
            BsMerchant merchant = merchantService.getMerchant(merNo);
            if (merchant == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的加油站");
            }
            List<BsAgentMer> collect = agentMerList.stream().filter(o -> o.getMerNo().equals(merNo)).collect(Collectors.toList());
            if (collect.isEmpty()) {
                agentMer = new BsAgentMer();
                agentMer.setAgentId(agent.getId());
                agentMer.setAgentName(agent.getName());
                agentMer.setAgentName(agent.getName());
                agentMer.setMerId(merchant.getId());
                agentMer.setMerNo(merchant.getMerNo());
                agentMer.setMerName(merchant.getMerName());
                agentMer.setStatus(1);
                agentMer.setCreateTime(new Date());
                agentMer.setUpdateTime(new Date());
                dataList.add(agentMer);
            }
        }
        agentMerService.addData(dataList);
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void unbindAgent(Long agentId, List<String> merNoArray) {
        // 删除代理商商户
        BsAgentMer agentMer = new BsAgentMer();
        agentMer.setStatus(0);
        BsAgentMerExample example = new BsAgentMerExample();
        example.createCriteria().andAgentIdEqualTo(agentId).andMerNoIn(merNoArray).andStatusNotEqualTo(0);
        agentMerMapper.updateByExampleSelective(agentMer, example);

        // 删除代理商下的工作人员商户
        List<Long> staffIdList = agentStaffService.getStaffList(agentId)
                .stream()
                .flatMapToLong(num -> LongStream.of(num.getId())).boxed()
                .collect(Collectors.toList());
        example.clear();
        example.createCriteria().andAgentStaffIdIn(staffIdList).andMerNoIn(merNoArray).andStatusNotEqualTo(0);
        agentMerMapper.updateByExampleSelective(agentMer, example);
    }

    @Override
    public List<AgentMerAllot> getAllotList(Map<String, Object> param) {
        return agentMerMapper.getAllotList(param);
    }

    @Override
    public List<BsAgentMer> getList(Map<String, Object> param) {
        BsAgentMerExample example = new BsAgentMerExample();
        BsAgentMerExample.Criteria criteria = example.createCriteria().andStatusNotEqualTo(0);

        if (MapUtils.getLong(param, "agentId") != null) {
            criteria.andAgentIdEqualTo(MapUtils.getLong(param, "agentId"));
        }

        if (MapUtils.getLong(param, "agentStaffId") != null) {
            criteria.andAgentStaffIdEqualTo(MapUtils.getLong(param, "agentStaffId"));
        }

        example.setOrderByClause("create_time desc");
        return agentMerMapper.selectByExample(example);
    }

    @Override
    public BsAgentMer getDetailByAgent(Long agentId, String merNo) {
        BsAgentMerExample example = new BsAgentMerExample();
        example.createCriteria().andAgentIdEqualTo(agentId).andMerNoEqualTo(merNo).andStatusNotEqualTo(0);
        List<BsAgentMer> list = agentMerMapper.selectByExample(example);
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public BsAgentMer getDetail(Long id) {
        return agentMerMapper.selectByPrimaryKey(id);
    }

    @Override
    public BsAgentMer getDetailByAgentStaff(Long agentStaffId, String merNo) {
        BsAgentMerExample example = new BsAgentMerExample();
        example.createCriteria().andAgentStaffIdEqualTo(agentStaffId).andMerNoEqualTo(merNo).andStatusNotEqualTo(0);
        List<BsAgentMer> list = agentMerMapper.selectByExample(example);
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public void assignStaff(Long staffId, List<String> merNoArray) {
        // 代理商工作人员
        BsAgentStaff staff = agentStaffService.getStaffDetailById(staffId);
        if (staff == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的代理商");
        }
        // 代理商
        BsAgent agent = agentService.getAgentById(staff.getAgentId());
        if (agent == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的代理商");
        }
        // 查询代理商工作人员的油站
        Map<String,Object> param = new HashMap<>();
        param.put("agentStaffId", agent.getId());
        List<BsAgentMer> agentMerList = getList(param);

        List<BsAgentMer> dataList = new ArrayList<>();
        BsAgentMer agentMer;
        for (String merNo : merNoArray) {
            // 查询门店
            BsMerchant merchant = merchantService.getMerchant(merNo);
            if (merchant == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的加油站");
            }
            List<BsAgentMer> collect = agentMerList.stream().filter(o -> o.getMerNo().equals(merNo)).collect(Collectors.toList());
            if (collect.isEmpty()) {
                agentMer = new BsAgentMer();
                agentMer.setAgentId(agent.getId());
                agentMer.setAgentName(agent.getName());
                agentMer.setAgentName(agent.getName());
                agentMer.setAgentStaffId(staff.getId());
                agentMer.setAgentStaffName(staff.getName());
                agentMer.setMerId(merchant.getId());
                agentMer.setMerNo(merchant.getMerNo());
                agentMer.setMerName(merchant.getMerName());
                agentMer.setStatus(1);
                agentMer.setCreateTime(new Date());
                agentMer.setUpdateTime(new Date());
                dataList.add(agentMer);
            }
        }
        agentMerService.addData(dataList);
    }

    @Override
    public void unbindStaff(Long staffId, List<String> merNoArray) {
        // 删除商户
        BsAgentMer agentMer = new BsAgentMer();
        agentMer.setStatus(0);
        BsAgentMerExample example = new BsAgentMerExample();
        example.createCriteria().andAgentStaffIdEqualTo(staffId).andMerNoIn(merNoArray).andStatusNotEqualTo(0);
        agentMerMapper.updateByExampleSelective(agentMer, example);
    }


}
