package com.hfkj.service.agent.impl;

import com.hfkj.common.utils.MD5Util;
import com.hfkj.common.utils.RedisUtil;
import com.hfkj.dao.BsAgentMapper;
import com.hfkj.entity.BsAgent;
import com.hfkj.entity.BsAgentExample;
import com.hfkj.entity.SecUser;
import com.hfkj.service.agent.BsAgentService;
import com.hfkj.service.sec.SecUserService;
import com.hfkj.sysenum.SecUserObjectTypeEnum;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @className: BsAgentServiceImpl
 * @author: HuRui
 * @date: 2024/2/28
 **/
@Service("agentService")
public class BsAgentServiceImpl implements BsAgentService {

    @Resource
    private BsAgentMapper agentMapper;

    @Resource
    private SecUserService secUserService;

    @Resource
    private RedisUtil redisUtil;

    // 缓存前缀
    private final static String CACHE_AGENT = "AGENT:";

    @Override
    public BsAgent editData(BsAgent data) {
        if (data.getId() == null) {
            data.setCreateTime(new Date());
            data.setUpdateTime(new Date());
            data.setStatus(1);
            agentMapper.insert(data);

            data.setAgentNo("A"+data.getId());
            agentMapper.updateByPrimaryKey(data);
        } else {
            data.setUpdateTime(new Date());
            agentMapper.updateByPrimaryKey(data);
        }
        return data;
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW,rollbackFor= {RuntimeException.class})
    public BsAgent createAgent(String loginName, BsAgent agent) throws Exception {
        editData(agent);

        SecUser secUser = new SecUser();
        secUser.setUserName(agent.getName());
        secUser.setLoginName(loginName);
        secUser.setPassword(MD5Util.encode("123456".getBytes()));
        secUser.setRoleId(5L);
        secUser.setObjectType(SecUserObjectTypeEnum.type4.getCode());
        secUser.setObjectId(agent.getId());
        secUser.setStatus(1);
        secUser.setCreateTime(new Date());
        secUser.setUpdateTime(new Date());
        secUserService.editUser(secUser);
        return agent;
    }

    @Override
    public BsAgent updateAgent(BsAgent agent) {
        editData(agent);

        redisUtil.set(CACHE_AGENT + agent.getAgentNo(), agent);
        return agent;
    }

    @Override
    public BsAgent getAgentById(Long id) {
        return agentMapper.selectByPrimaryKey(id);
    }

    @Override
    public BsAgent getAgentByAgentNo(String agentNo) {
        Object obj = redisUtil.get(CACHE_AGENT + agentNo);
        if (obj != null) {
            return (BsAgent) obj;
        }
        BsAgentExample example = new BsAgentExample();
        example.createCriteria().andAgentNoEqualTo(agentNo).andStatusNotEqualTo(0);
        List<BsAgent> list = agentMapper.selectByExample(example);
        if (!list.isEmpty()) {
            BsAgent agent = list.get(0);
            redisUtil.set(CACHE_AGENT + agentNo, agent);
            return agent;
        }
        return null;
    }

    @Override
    public List<BsAgent> getAgentList(Map<String, Object> param) {
        BsAgentExample example = new BsAgentExample();
        BsAgentExample.Criteria criteria = example.createCriteria().andStatusNotEqualTo(0);

        if (StringUtils.isNotBlank(MapUtils.getString(param, "agentNo"))) {
            criteria.andAgentNoLike("%" + MapUtils.getString(param, "agentNo") + "%");
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "agentName"))) {
            criteria.andNameLike("%" + MapUtils.getString(param, "agentName") + "%");
        }

        example.setOrderByClause("create_time desc");
        return agentMapper.selectByExample(example);
    }

}
