package com.hfkj.service.impl;

import com.hfkj.common.utils.DateUtil;
import com.hfkj.config.SpPrinterConfig;
import com.hfkj.config.SpPrinterTemplate;
import com.hfkj.dao.BsDeviceMapper;
import com.hfkj.entity.BsDevice;
import com.hfkj.entity.BsDeviceExample;
import com.hfkj.entity.BsGasOrder;
import com.hfkj.service.BsDeviceService;
import com.hfkj.sysenum.DeviceTypeEnum;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("deviceService")
public class BsDeviceServiceImpl implements BsDeviceService {

    @Resource
    private BsDeviceMapper deviceMapper;

    @Override
    public void editDevice(BsDevice device) {
        if (device.getId() == null) {
            device.setCreateTime(new Date());
            device.setUpdateTime(new Date());
            device.setStatus(1);
            deviceMapper.insert(device);
        } else {
            device.setUpdateTime(new Date());
            deviceMapper.updateByPrimaryKey(device);
        }
    }

    @Override
    public BsDevice getDetailById(Long deviceId) {
        return deviceMapper.selectByPrimaryKey(deviceId);
    }

    @Override
    public List<BsDevice> getDeviceByMer(Long merId) {
        BsDeviceExample example = new BsDeviceExample();
        example.createCriteria().andMerIdEqualTo(merId).andStatusNotEqualTo(0);
        example.setOrderByClause("create_time desc");
        return deviceMapper.selectByExample(example);
    }

    @Override
    public List<BsDevice> getDeviceList(Map<String, Object> param) {
        BsDeviceExample example = new BsDeviceExample();
        BsDeviceExample.Criteria criteria = example.createCriteria().andStatusNotEqualTo(0);

        if (MapUtils.getInteger(param, "type") != null) {
            criteria.andTypeEqualTo(MapUtils.getInteger(param, "type"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "merNo"))) {
            criteria.andMerNoEqualTo(MapUtils.getString(param, "merNo"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "merName"))) {
            criteria.andMerNameLike("%"+MapUtils.getString(param, "merName")+"%");
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "deviceName"))) {
            criteria.andDeviceNameLike("%" + MapUtils.getString(param, "deviceName") + "%");
        }

        example.setOrderByClause("create_time desc");
        return deviceMapper.selectByExample(example);
    }

    @Override
    public void printGasOrder(Long merId, BsGasOrder gasOrder, boolean makeUp) {
        // 查询加油站打印机
        List<BsDevice> deviceList = getDeviceByMer(merId);
        for (BsDevice device : deviceList) {
            Map<String, Object> receiptMap = new HashMap<>();
            receiptMap.put("receiptTop", device.getReceiptTop());
            receiptMap.put("receiptSource", device.getReceiptSource());
            receiptMap.put("receiptBottom", device.getReceiptBottom());

            if (device.getType().equals(DeviceTypeEnum.type1.getType())) {
                new Thread(() -> {
                    try {
                        SpPrinterConfig sp = new SpPrinterConfig();
                        sp.print(device.getDeviceSn(),
                                SpPrinterTemplate.oilCashierStubTemp(
                                        gasOrder.getMerName(),
                                        gasOrder.getOrderNo(),
                                        DateUtil.date2String(gasOrder.getPayTime(), "yyyy-MM-dd HH:mm:ss"),
                                        gasOrder.getUserPhone(),
                                        gasOrder.getGasGunNo(),
                                        gasOrder.getGasOilNo(),
                                        gasOrder.getGasOilLiters().toString(),
                                        gasOrder.getGasRefuelPrice().toString(),
                                        receiptMap,
                                        makeUp
                                ), 1);

                        sp.print(device.getDeviceSn(),
                                SpPrinterTemplate.oilClientStubTemp(
                                        gasOrder.getMerName(),
                                        gasOrder.getOrderNo(),
                                        DateUtil.date2String(gasOrder.getPayTime(), "yyyy-MM-dd HH:mm:ss"),
                                        gasOrder.getUserPhone(),
                                        gasOrder.getGasGunNo(),
                                        gasOrder.getGasOilNo(),
                                        gasOrder.getGasOilLiters().toString(),
                                        gasOrder.getGasRefuelPrice().toString(),
                                        receiptMap,
                                        makeUp
                                ), 1);

                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }).start();
            }
        }

    }
}
