package com.hfkj.service.order;

import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.AESEncodeUtil;
import com.hfkj.entity.*;
import com.hfkj.model.GasPayPriceModel;
import com.hfkj.model.order.OrderChildModel;
import com.hfkj.service.BsMerchantService;
import com.hfkj.service.agent.BsAgentMerService;
import com.hfkj.service.gas.BsGasClassGroupTaskService;
import com.hfkj.service.gas.BsGasOrderService;
import com.hfkj.service.gas.BsGasService;
import com.hfkj.service.gas.BsGasStaffService;
import com.hfkj.service.sec.SecDictionaryService;
import com.hfkj.sysenum.MerchantStatusEnum;
import com.hfkj.sysenum.gas.OrderOilStatus;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 创建订单业务
 * @className: OrderCreateService
 * @author: HuRui
 * @date: 2024/6/7
 **/
@Component
public class OrderCreateService {
    Logger log = LoggerFactory.getLogger(OrderCreateService.class);

    @Resource
    private BsMerchantService merchantService;
    @Resource
    private SecDictionaryService secDictionaryService;
    @Resource
    private BsGasService gasService;
    @Resource
    private BsGasOrderService gasOrderService;
    @Resource
    private BsGasClassGroupTaskService gasClassGroupTaskService;
    @Resource
    private BsAgentMerService agentMerService;
    @Resource
    private BsGasStaffService gasStaffService;

    /**
     * 加油业务
     * @param order  订单
     * @param orderDeduction 优惠
     * @param orderChild   商品订单
     * @return
     */
    public OrderChildModel oilHandle(BsOrder order, BsOrderDeduction orderDeduction, OrderChildModel orderChild) {
        // 查询油品类型
        SecDictionary gasOilType = secDictionaryService.getDictionary("OIL_NO", orderChild.getGasOilNo());
        if (gasOilType == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知油品类型");
        }
        // 查询油站
        BsMerchant merchant = merchantService.getMerchant(orderChild.getMerNo());
        if (merchant == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的油站");
        }
        if (!merchant.getStatus().equals(MerchantStatusEnum.status1.getNumber())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "当前油站暂停交易");
        }

        // 计算价格
        GasPayPriceModel priceModel = gasService.refuelPriceCompute(orderChild.getProductPrice(), merchant.getMerNo(), orderChild.getGasOilNo(), orderChild.getAgentMerId());

        BsGasOrder gasOrder = new BsGasOrder();
        gasOrder.setMerNo(merchant.getMerNo());
        gasOrder.setChannelType(merchant.getSourceType());
        gasOrder.setOrderNo(orderChild.getOrderNo());
        gasOrder.setOrderChildNo(orderChild.getChildOrderNo());
        gasOrder.setUserId(order.getUserId());
        gasOrder.setUserPhone(order.getUserPhone());
        gasOrder.setMerId(merchant.getId());
        gasOrder.setMerName(merchant.getMerName());
        gasOrder.setMerAddress(merchant.getAddress());
        gasOrder.setStatus(OrderOilStatus.STATUS1.getNumber());

        gasOrder.setDeductionCouponPrice(new BigDecimal("0"));
        gasOrder.setDeductionOilPrice(priceModel.getTotalPreferences());
        gasOrder.setPayablePrice(priceModel.getPayPrice());
        gasOrder.setTotalDeductionPrice(gasOrder.getDeductionOilPrice().add(gasOrder.getDeductionCouponPrice()));

        // 油价信息
        gasOrder.setGasRefuelPrice(orderChild.getProductPrice());
        gasOrder.setGasOilNo(orderChild.getGasOilNo());
        gasOrder.setGasGunNo(orderChild.getGasGunNo());
        gasOrder.setGasOilType(Integer.parseInt(gasOilType.getExt1()));
        gasOrder.setGasPriceGun(priceModel.getPriceGun());
        gasOrder.setGasPriceVip(priceModel.getPriceVip());
        gasOrder.setGasPriceOfficial(priceModel.getPriceOfficial());
        gasOrder.setGasPricePlatform(priceModel.getPricePlatform());
        gasOrder.setGasOilLiters(priceModel.getOilLiters());
        gasOrder.setGasDiscount(priceModel.getDiscount());
        gasOrder.setGasOilSubsidy(priceModel.getPreferentialMargin());
        gasOrder.setGasLitersPreferences(priceModel.getLitersPreferences());
        gasOrder.setGasPricePreferences(priceModel.getPricePreferences());

        // 代理、业务员
        if (orderChild.getAgentMerId() != null) {
            // 查询代理
            BsAgentMer agentMer = agentMerService.getDetail(orderChild.getAgentMerId());
            if (agentMer == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的代理");
            }
            gasOrder.setAgentId(agentMer.getAgentId());
            gasOrder.setAgentName(agentMer.getAgentName());
            gasOrder.setAgentStaffId(agentMer.getAgentStaffId());
            gasOrder.setAgentStaffName(agentMer.getAgentStaffName());
        }

        // 加油员
        if (orderChild.getGasStaffId() != null) {
            // 查询员工
            BsGasStaff gasStaff = gasStaffService.getStaffDetailById(orderChild.getGasStaffId());
            if (gasStaff == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知加油员");
            }
            if (gasStaff.getMerId().equals(merchant.getId())) {
                gasOrder.setGasStaffId(gasStaff.getId());
                gasOrder.setGasStaffName(gasStaff.getName());
            }
        }

        // 油站是否开启班组
        BsGasClassGroupTask groupTask = gasClassGroupTaskService.getCurrentTaskByMerId(merchant.getId());
        if (groupTask != null) {
            gasOrder.setGasClassGroupId(groupTask.getGasClassGroupId());
            gasOrder.setGasClassGroupName(groupTask.getGasClassGroupName());
            gasOrder.setGasClassGroupTaskId(groupTask.getId());
        }

        gasOrderService.addGasOrder(gasOrder);

        orderChild.setMerNo(merchant.getMerNo());
        orderChild.setProductName(merchant.getMerName());
        orderChild.setProductImg(merchant.getMerLogo());
        orderChild.setProductSpecName("默认");
        orderChild.setProductCount(1);
        orderChild.setProductActualPrice(gasOrder.getPayablePrice());
        return orderChild;
    }

}
