package com.hfkj.service.order.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hfkj.dao.BsOrderSettleMapper;
import com.hfkj.entity.BsMerchantPayConfig;
import com.hfkj.entity.BsOrderSettle;
import com.hfkj.entity.BsOrderSettleExample;
import com.hfkj.entity.BsOrderSettleLedger;
import com.hfkj.pay.HuiPayService;
import com.hfkj.service.BsMerchantPayConfigService;
import com.hfkj.service.order.BsOrderSettleLedgerService;
import com.hfkj.service.order.BsOrderSettleService;
import com.hfkj.sysenum.order.OrderSettleLedgerStatusEnum;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @className: BsOrderSettleServiceImpl
 * @author: HuRui
 * @date: 2024/6/24
 **/
@Service("BsOrderSettleService")
public class BsOrderSettleServiceImpl implements BsOrderSettleService {
    @Resource
    private BsOrderSettleMapper orderSettleMapper;
    @Resource
    private BsOrderSettleLedgerService orderSettleLedgerService;
    @Resource
    private BsMerchantPayConfigService merchantPayConfigService;

    @Override
    public void editData(BsOrderSettle data) {
        data.setUpdateTime(new Date());
        if (data.getId() == null) {
            data.setCreateTime(new Date());
            orderSettleMapper.insert(data);
        } else {
            orderSettleMapper.updateByPrimaryKey(data);
        }
    }

    @Override
    public BsOrderSettle getData(String orderNo) {
        BsOrderSettleExample example = new BsOrderSettleExample();
        example.createCriteria().andOrderNoEqualTo(orderNo);
        List<BsOrderSettle> list = orderSettleMapper.selectByExample(example);
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public void separate(BsOrderSettle orderSettle) {
        // 查询商户配置
        BsMerchantPayConfig payConfig = merchantPayConfigService.getConfig(orderSettle.getMerNo());
        if (payConfig != null && StringUtils.isNotBlank(payConfig.getLedgerAccountData())) {
            List<BsOrderSettleLedger> settleLedgerList = new ArrayList<>();
            // 分账接收方数据
            JSONArray ledgerAccountData = JSONArray.parseArray(payConfig.getLedgerAccountData());
            for (Object obj : ledgerAccountData) {
                JSONObject ledgerAccount = (JSONObject) obj;
                BsOrderSettleLedger settleLedger = new BsOrderSettleLedger();
                settleLedger.setOrderSettleId(orderSettle.getId());
                settleLedger.setSharingAmount(orderSettle.getSettleAmount());
                settleLedger.setRatio(ledgerAccount.getBigDecimal("receiverRatio"));
                settleLedger.setReceiverMerNo(ledgerAccount.getString("receiverMerNo"));
                settleLedger.setReceiverMerName(ledgerAccount.getString("receiverMerName"));
                settleLedger.setReceiverAmount(settleLedger.getSharingAmount().multiply(settleLedger.getRatio().divide(new BigDecimal("100"))));
                settleLedger.setStatus(0);
                orderSettleLedgerService.editData(settleLedger);

                settleLedgerList.add(settleLedger);
            }

            // 请求分账
            JSONObject separate = HuiPayService.profitSharingSeparate(orderSettle, settleLedgerList);
            if (separate.getString("return_code").equals("000000")) {
                orderSettle.setLedgerStatus(OrderSettleLedgerStatusEnum.status2.getCode());
                editData(orderSettle);
            }
        }

    }

    @Override
    public List<BsOrderSettle> getList(Map<String, Object> param) {
        BsOrderSettleExample example = new BsOrderSettleExample();
        BsOrderSettleExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getInteger(param, "ledgerStatus") != null) {
            criteria.andLedgerStatusEqualTo(MapUtils.getInteger(param, "ledgerStatus"));
        }

        return orderSettleMapper.selectByExample(example);
    }
}
